@@ -4,7 +4,7 @@
 
 struct Param;
 
-int mars_mainBody(Param &params, Strings &files, Strings &libmodules);
+int mars_tryMain(Param &params, Strings &files);
 
 void parseTransitionOption(Param &params, const char *name);
 void parsePreviewOption(Param &params, const char *name);
