@@ -155,8 +155,11 @@ Where:\n\
   -conf=<filename>  use config file at filename\n\
   -cov              do code coverage analysis\n\
   -cov=ctfe         include code executed during CTFE in coverage report\n\
-  -cov=<nnn>        require at least nnn%% code coverage\n\
-  -D                generate documentation\n\
+  -cov=<nnn>        require at least nnn%% code coverage\n"
+#if 0
+"  -cpp=<filename>   use filename as the name of the C preprocessor to use for ImportC files\n"
+#endif
+"  -D                generate documentation\n\
   -Dd<directory>    write documentation file to directory\n\
   -Df<filename>     write documentation file to filename\n\
   -d                silently allow deprecated features and symbols\n\
@@ -170,6 +173,8 @@ Where:\n\
                     set default library to name\n\
   -deps             print module dependencies (imports/file/version/debug/lib)\n\
   -deps=<filename>  write module dependencies to filename (only imports)\n\
+  -dllimport=<value>\n\
+                    Windows only: select symbols to dllimport (none/defaultLibsOnly/all)\n\
   -extern-std=<standard>\n\
                     set C++ name mangling compatibility with <standard>\n"
 #if 0
@@ -192,16 +197,20 @@ Where:\n\
   -Hd=<directory>   write 'header' file to directory\n\
   -Hf=<filename>    write 'header' file to filename\n\
   -HC[=[silent|verbose]]\n\
-                    generate C++ 'header' file\n"
+                    write C++ 'header' equivalent to stdout\n"
 #if 0
 "  -HC=[?|h|help]    list available modes for C++ 'header' file generation\n"
 #endif
 "  -HCd=<directory>  write C++ 'header' file to directory\n\
-  -HCf=<filename>   write C++ 'header' file to filename\n\
+  -HCf=<filename>   write C++ 'header' file to filename instead of stdout\n\
   --help            print help and exit\n\
   -I=<directory>    look for imports also in directory\n\
   -i[=<pattern>]    include imported modules in the compilation\n\
-  -ignore           ignore unsupported pragmas\n\
+  -identifiers=<table>\n\
+                    specify the non-ASCII tables for D identifiers\n\
+  -identifiers-importc=<table>\n\
+                    specify the non-ASCII tables for ImportC identifiers\n\
+  -ignore           deprecated flag, unsupported pragmas are always ignored now\n\
   -inline           do function inlining\n\
   -J=<directory>    look for string imports also in directory\n\
   -L=<linkerflag>   pass linkerflag to link\n\
@@ -245,7 +254,7 @@ Where:\n\
 #if 0
 "  -profile=gc       profile runtime allocations\n"
 #endif
-"  -release          compile release version\n\
+"  -release          contracts and asserts are not emitted, and bounds checking is performed only in @safe functions\n\
   -revert=<name>    revert language change identified by 'name'\n\
   -revert=[h|help|?]\n\
                     list all revertable language changes\n\
@@ -272,12 +281,13 @@ Where:\n\
   -version=<level>  compile in version code >= level\n\
   -version=<ident>  compile in version code identified by ident\n\
   -vgc              list all gc allocations including hidden ones\n\
+  -visibility=<value>\n\
+                    default visibility of symbols (default/hidden/public)\n\
   -vtemplates=[list-instances]\n\
                     list statistics on template instantiations\n\
   -vtls             list all variables going into thread local storage\n\
   -w                warnings as errors (compilation will halt)\n\
   -wi               warnings as messages (compilation will continue)\n\
-  -wo               warnings about use of obsolete features (compilation will continue)\n\
   -X                generate JSON file\n\
   -Xf=<filename>    write JSON file to filename\n\
   -Xcc=<driverflag> pass driverflag to linker driver (cc)\n",
@@ -491,6 +501,11 @@ void translateArgs(const llvm::SmallVectorImpl<const char *> &ldmdArgs,
        * -cov
        * -shared
        */
+      else if (startsWith(p + 1, "visibility=")) {
+        ldcArgs.push_back(concat("-fvisibility=", p + 12));
+      }
+      /* -dllimport
+       */
       else if (strcmp(p + 1, "dylib") == 0) {
         ldcArgs.push_back("-shared");
       } else if (strcmp(p + 1, "fIBT") == 0) {
@@ -575,8 +590,7 @@ void translateArgs(const llvm::SmallVectorImpl<const char *> &ldmdArgs,
         const char *c = p + 6;
         if (strcmp(c, "?") == 0 || strcmp(c, "h") == 0 ||
             strcmp(c, "help") == 0) {
-          execute({ldcPath.c_str(), "-mcpu=help"});
-          exit(EXIT_SUCCESS);
+          ldcArgs.push_back("-mcpu=help");
         } else if (strcmp(c, "baseline") == 0) {
           // ignore
         } else if (strcmp(c, "avx") == 0) {
@@ -625,7 +639,9 @@ void translateArgs(const llvm::SmallVectorImpl<const char *> &ldmdArgs,
         ldcArgs.push_back("-enable-inlining");
         ldcArgs.push_back("-Hkeep-all-bodies");
       }
-      /* -dip25
+      /* -identifiers-importc
+       * -identifiers
+       * -dip25
        * -dip1000
        * -dip1008
        */
@@ -741,13 +757,7 @@ void translateArgs(const llvm::SmallVectorImpl<const char *> &ldmdArgs,
       }
     } else {
       const auto ext = ls::path::extension(p);
-      if (
-#if LDC_LLVM_VER >= 1300
-        ext.equals_insensitive(".exe")
-#else
-        ext.equals_lower(".exe")
-#endif
-          ) {
+      if (ext.equals_insensitive(".exe")) {
         // should be for Windows targets only
         ldcArgs.push_back(concat("-of=", p));
         continue;
