@@ -65,11 +65,7 @@
 #include "llvm/Support/ManagedStatic.h"
 #include "llvm/Support/Path.h"
 #include "llvm/Support/StringSaver.h"
-#if LDC_LLVM_VER >= 1400
 #include "llvm/MC/TargetRegistry.h"
-#else
-#include "llvm/Support/TargetRegistry.h"
-#endif
 #include "llvm/Support/TargetSelect.h"
 #include "llvm/Target/TargetMachine.h"
 #if LDC_MLIR_ENABLED
@@ -91,6 +87,7 @@ void gendocfile(Module *m);
 // In dmd/mars.d
 void generateJson(Modules *modules);
 
+using namespace dmd;
 using namespace opts;
 
 static StringsAdapter impPathsStore("I", global.params.imppath);
@@ -188,6 +185,10 @@ void tryParse(const llvm::SmallVectorImpl<const char *> &args, size_t i,
 }
 
 bool tryParseLowmem(const llvm::SmallVectorImpl<const char *> &args) {
+#if LDC_LLVM_VER >= 1800
+  #define startswith starts_with
+#endif
+
   bool lowmem = false;
   for (size_t i = 1; i < args.size(); ++i) {
     if (args::isRunArg(args[i]))
@@ -205,6 +206,10 @@ bool tryParseLowmem(const llvm::SmallVectorImpl<const char *> &args) {
     }
   }
   return lowmem;
+
+#if LDC_LLVM_VER >= 1800
+  #undef startswith
+#endif
 }
 
 const char *
@@ -385,10 +390,8 @@ void parseCommandLine(Strings &sourceFiles) {
   }
 
 #if _WIN32
-  const auto toWinPaths = [](Strings *paths) {
-    if (!paths)
-      return;
-    for (auto &path : *paths)
+  const auto toWinPaths = [](Strings &paths) {
+    for (auto &path : paths)
       path = opts::dupPathString(path).ptr;
   };
   toWinPaths(global.params.imppath);
@@ -555,29 +558,11 @@ void parseCommandLine(Strings &sourceFiles) {
   global.params.dihdr.fullOutput = opts::hdrKeepAllBodies;
   global.params.disableRedZone = opts::disableRedZone();
 
-  // Passmanager selection options depend on LLVM version
-#if LDC_LLVM_VER < 1400
-  // LLVM < 14 only supports the legacy passmanager
-  if (!opts::isUsingLegacyPassManager()) {
-    error(Loc(), "LLVM version 13 or below only supports --passmanager=legacy");
-  }
-#endif
-#if LDC_LLVM_VER >= 1500
-  // LLVM >= 15 only supports the new passmanager
-  if (opts::isUsingLegacyPassManager()) {
-    error(Loc(), "LLVM version 15 or above only supports --passmanager=new");
-  }
-#endif
-
+  // enforce opaque IR pointers
 #if LDC_LLVM_VER >= 1700
-  if (!opts::enableOpaqueIRPointers)
-    error(Loc(),
-          "LLVM version 17 or above only supports --opaque-pointers=true");
-#elif LDC_LLVM_VER >= 1500
-  getGlobalContext().setOpaquePointers(opts::enableOpaqueIRPointers);
-#elif LDC_LLVM_VER >= 1400
-  if (opts::enableOpaqueIRPointers)
-    getGlobalContext().enableOpaquePointers();
+  // supports opaque IR pointers only
+#else
+  getGlobalContext().setOpaquePointers(true);
 #endif
 }
 
@@ -779,6 +764,9 @@ void registerPredefinedTargetVersions() {
     VersionCondition::addPredefinedGlobalIdent("LoongArch64");
     registerPredefinedFloatABI("LoongArch_SoftFloat", "LoongArch_HardFloat");
     break;
+  case llvm::Triple::xtensa:
+    VersionCondition::addPredefinedGlobalIdent("Xtensa");
+    break;
 #endif // LDC_LLVM_VER >= 1600
   default:
     warning(Loc(), "unknown target CPU architecture: %s",
@@ -845,16 +833,20 @@ void registerPredefinedTargetVersions() {
     if (triple.getEnvironment() == llvm::Triple::Android) {
       VersionCondition::addPredefinedGlobalIdent("Android");
       VersionCondition::addPredefinedGlobalIdent("CRuntime_Bionic");
+      VersionCondition::addPredefinedGlobalIdent("CppRuntime_LLVM");
+      VersionCondition::addPredefinedGlobalIdent("CppRuntime_Clang"); // legacy
     } else if (triple.isMusl()) {
       VersionCondition::addPredefinedGlobalIdent("CRuntime_Musl");
-      VersionCondition::addPredefinedGlobalIdent("CppRuntime_Gcc");
+      VersionCondition::addPredefinedGlobalIdent("CppRuntime_GNU");
+      VersionCondition::addPredefinedGlobalIdent("CppRuntime_Gcc"); // legacy
       // use libunwind for backtraces
       VersionCondition::addPredefinedGlobalIdent("DRuntime_Use_Libunwind");
     } else if (global.params.isUClibcEnvironment) {
       VersionCondition::addPredefinedGlobalIdent("CRuntime_UClibc");
     } else {
       VersionCondition::addPredefinedGlobalIdent("CRuntime_Glibc");
-      VersionCondition::addPredefinedGlobalIdent("CppRuntime_Gcc");
+      VersionCondition::addPredefinedGlobalIdent("CppRuntime_GNU");
+      VersionCondition::addPredefinedGlobalIdent("CppRuntime_Gcc"); // legacy
     }
     break;
   case llvm::Triple::Haiku:
@@ -867,7 +859,8 @@ void registerPredefinedTargetVersions() {
     VersionCondition::addPredefinedGlobalIdent(
         "darwin"); // For backwards compatibility.
     VersionCondition::addPredefinedGlobalIdent("Posix");
-    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Clang");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_LLVM");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Clang"); // legacy
     break;
   case llvm::Triple::FreeBSD:
     VersionCondition::addPredefinedGlobalIdent("FreeBSD");
@@ -878,7 +871,8 @@ void registerPredefinedTargetVersions() {
       warning(Loc(), "FreeBSD major version not specified in target triple");
     }
     VersionCondition::addPredefinedGlobalIdent("Posix");
-    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Clang");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_LLVM");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Clang"); // legacy
     break;
   case llvm::Triple::Solaris:
     VersionCondition::addPredefinedGlobalIdent("Solaris");
@@ -888,7 +882,8 @@ void registerPredefinedTargetVersions() {
   case llvm::Triple::DragonFly:
     VersionCondition::addPredefinedGlobalIdent("DragonFlyBSD");
     VersionCondition::addPredefinedGlobalIdent("Posix");
-    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Gcc");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_GNU");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Gcc"); // legacy
     break;
   case llvm::Triple::NetBSD:
     VersionCondition::addPredefinedGlobalIdent("NetBSD");
@@ -897,7 +892,8 @@ void registerPredefinedTargetVersions() {
   case llvm::Triple::OpenBSD:
     VersionCondition::addPredefinedGlobalIdent("OpenBSD");
     VersionCondition::addPredefinedGlobalIdent("Posix");
-    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Gcc");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_GNU");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Gcc"); // legacy
     break;
   case llvm::Triple::AIX:
     VersionCondition::addPredefinedGlobalIdent("AIX");
@@ -906,22 +902,34 @@ void registerPredefinedTargetVersions() {
   case llvm::Triple::IOS:
     VersionCondition::addPredefinedGlobalIdent("iOS");
     VersionCondition::addPredefinedGlobalIdent("Posix");
-    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Clang");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_LLVM");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Clang"); // legacy
     break;
   case llvm::Triple::TvOS:
     VersionCondition::addPredefinedGlobalIdent("TVOS");
     VersionCondition::addPredefinedGlobalIdent("Posix");
-    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Clang");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_LLVM");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Clang"); // legacy
     break;
   case llvm::Triple::WatchOS:
     VersionCondition::addPredefinedGlobalIdent("WatchOS");
     VersionCondition::addPredefinedGlobalIdent("Posix");
-    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Clang");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_LLVM");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Clang"); // legacy
     break;
   case llvm::Triple::WASI:
     VersionCondition::addPredefinedGlobalIdent("WASI");
     VersionCondition::addPredefinedGlobalIdent("CRuntime_WASI");
     break;
+  case llvm::Triple::Emscripten:
+    VersionCondition::addPredefinedGlobalIdent("Emscripten");
+    // Emscripten uses musl and libc++, so mimic a musl Linux platform:
+    VersionCondition::addPredefinedGlobalIdent("linux");
+    VersionCondition::addPredefinedGlobalIdent("Posix");
+    VersionCondition::addPredefinedGlobalIdent("CRuntime_Musl");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_LLVM");
+    VersionCondition::addPredefinedGlobalIdent("CppRuntime_Clang"); // legacy
+    break;
   default:
     if (triple.getEnvironment() == llvm::Triple::Android) {
       VersionCondition::addPredefinedGlobalIdent("Android");
@@ -1033,16 +1041,6 @@ void registerPredefinedVersions() {
     VersionCondition::addPredefinedGlobalIdent("LDC_ThreadSanitizer");
   }
 
-  // Set a version identifier for whether opaque pointers are enabled or not. (needed e.g. for intrinsic mangling)
-#if LDC_LLVM_VER >= 1700
-  // Since LLVM 17, IR pointers are always opaque.
-  VersionCondition::addPredefinedGlobalIdent("LDC_LLVM_OpaquePointers");
-#elif LDC_LLVM_VER >= 1400
-  if (!getGlobalContext().supportsTypedPointers()) {
-    VersionCondition::addPredefinedGlobalIdent("LDC_LLVM_OpaquePointers");
-  }
-#endif
-
 // Expose LLVM version to runtime
 #define STR(x) #x
 #define XSTR(x) STR(x)
@@ -1145,9 +1143,6 @@ int cppmain() {
     fatal();
   }
 
-  global.compileEnv.previewIn = global.params.previewIn;
-  global.compileEnv.ddocOutput = global.params.ddoc.doOutput;
-
   if (opts::fTimeTrace) {
     initializeTimeTrace(opts::fTimeTraceGranularity, 0, opts::allArguments[0]);
   }
@@ -1232,8 +1227,7 @@ int cppmain() {
   int status;
   {
     TimeTraceScope timeScope("ExecuteCompiler");
-    Strings libmodules;
-    status = mars_mainBody(global.params, files, libmodules);
+    status = mars_tryMain(global.params, files);
   }
 
   // try to remove the temp objects dir if created for -cleanup-obj
