@@ -26,11 +26,8 @@ union CTFloatUnion {
 APFloat parseLiteral(const llvm::fltSemantics &semantics, const char *literal,
                      bool *isOutOfRange = nullptr) {
   APFloat ap(semantics, APFloat::uninitialized);
-  auto r =
-#if LDC_LLVM_VER >= 1000
-      llvm::cantFail
-#endif
-      (ap.convertFromString(literal, APFloat::rmNearestTiesToEven));
+  auto r = llvm::cantFail(
+      ap.convertFromString(literal, APFloat::rmNearestTiesToEven));
   if (isOutOfRange) {
     *isOutOfRange = (r & (APFloat::opOverflow | APFloat::opUnderflow)) != 0;
   }
@@ -109,8 +106,8 @@ real_t CTFloat::fromAPFloat(const APFloat &src_) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-real_t CTFloat::parse(const char *literal, bool *isOutOfRange) {
-  const APFloat ap = parseLiteral(*apSemantics, literal, isOutOfRange);
+real_t CTFloat::parse(const char *literal, bool &isOutOfRange) {
+  const APFloat ap = parseLiteral(*apSemantics, literal, &isOutOfRange);
   return fromAPFloat(ap);
 }
 
@@ -128,19 +125,25 @@ bool CTFloat::isFloat64LiteralOutOfRange(const char *literal) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-int CTFloat::sprint(char *str, char fmt, real_t x) {
+int CTFloat::sprint(char *str, size_t str_buf_length, char fmt, real_t x) {
   assert(fmt == 'g' || fmt == 'a' || fmt == 'A');
   const bool uppercase = fmt == 'A';
 
   // We try to keep close to C printf and handle a few divergences of the LLVM
   // to-string utility functions.
 
+  // For simplicity later on, do not accept values of str_buf_length below 5.
+  if (str_buf_length < 5)
+    return 0;
+
   if (isNaN(x)) {
-    int length = 0;
+    size_t length = 0;
     if (copysign(one, x) != one) {
       str[0] = '-';
       ++length;
     }
+    if (length > str_buf_length-4)
+      length = str_buf_length-4;
     memcpy(str + length, uppercase ? "NAN" : "nan", 3);
     length += 3;
     str[length] = 0;
@@ -148,11 +151,13 @@ int CTFloat::sprint(char *str, char fmt, real_t x) {
   }
 
   if (isInfinity(x)) { // incl. -inf
-    int length = 0;
+    size_t length = 0;
     if (x < 0) {
       str[0] = '-';
       ++length;
     }
+    if (length > str_buf_length-4)
+      length = str_buf_length-4;
     memcpy(str + length, uppercase ? "INF" : "inf", 3);
     length += 3;
     str[length] = 0;
@@ -164,9 +169,12 @@ int CTFloat::sprint(char *str, char fmt, real_t x) {
     APFloat ap(0.0);
     toAPFloat(x, ap);
 
-    int length =
+    size_t length =
         ap.convertToHexString(str, 0, uppercase, APFloat::rmNearestTiesToEven);
 
+    if (length > str_buf_length-1)
+      length = str_buf_length-1;
+
     // insert a '+' prefix for non-negative exponents (incl. 0) as visual aid
     const char p = uppercase ? 'P' : 'p';
     for (int i = length - 2; i >= 0; --i) {
@@ -193,20 +201,20 @@ int CTFloat::sprint(char *str, char fmt, real_t x) {
   // * powl(2.5L, 2.5L) = 9.882117688... => `9.88211` (not 9.88212)
   // * 1e-300L => `9.99999e-301`
 #ifdef _MSC_VER
-  int length = sprintf(str, "%g", static_cast<double>(x));
+  size_t length = snprintf(str, str_buf_length-1, "%g", static_cast<double>(x));
 #else
-  int length = sprintf(str, "%Lg", x);
+  size_t length = snprintf(str, str_buf_length-1, "%Lg", x);
 #endif
 
   // 1 => 1.0 to distinguish from integers
   bool needsFPSuffix = true;
-  for (int i = 0; i < length; ++i) {
+  for (size_t i = 0; i < length; ++i) {
     if (str[i] != '-' && !isdigit(str[i])) {
       needsFPSuffix = false;
       break;
     }
   }
-  if (needsFPSuffix) {
+  if (needsFPSuffix && (length < str_buf_length-3)) {
     memcpy(str + length, ".0", 2);
     length += 2;
   }
