@@ -17,7 +17,14 @@
 #include "gen/irstate.h"
 #include "gen/llvm.h"
 #include "gen/tollvm.h"
+#if LDC_LLVM_VER < 1700
 #include "llvm/ADT/Optional.h"
+#else
+#include <optional>
+namespace llvm {
+template <typename T> using Optional = std::optional<T>;
+}
+#endif
 
 class Dsymbol;
 class Type;
