@@ -24,6 +24,7 @@
 #include <string>
 
 // from SPIRVInternal.h
+#if LDC_LLVM_VER < 1900
 #define SPIR_TARGETTRIPLE32 "spir-unknown-unknown"
 #define SPIR_TARGETTRIPLE64 "spir64-unknown-unknown"
 #define SPIR_DATALAYOUT32                                                      \
@@ -38,6 +39,18 @@
   "-v32:32:32-v48:64:64-v64:64:64-v96:128:128"                                 \
   "-v128:128:128-v192:256:256-v256:256:256"                                    \
   "-v512:512:512-v1024:1024:1024"
+#else
+#define SPIR_TARGETTRIPLE32 "spirv-unknown-unknown"
+#define SPIR_TARGETTRIPLE64 "spirv64-unknown-unknown"
+#define SPIR_DATALAYOUT32                                                      \
+  "e-p:32:32-i64:64-v16:16-v24:32-v32:32-v48:64"                               \
+  "-v96:128-v192:256-v256:256-v512:512-v1024:1024-G1"
+#define SPIR_DATALAYOUT64                                                      \
+  "e-i64:64-v16:16-v24:32-v32:32-v48:64-v96:128"                               \
+  "-v192:256-v256:256-v512:512-v1024:1024-G1"
+#endif
+
+using namespace dmd;
 
 namespace {
 class TargetOCL : public DComputeTarget {
@@ -157,7 +170,6 @@ public:
       ss << "uchar";
     else if (ty == TY::Tvector) {
       TypeVector *vec = static_cast<TypeVector *>(t);
-      auto size = vec->size(Loc());
       auto basety = vec->basetype->ty;
       if (basety == TY::Tint8)
         ss << "char";
@@ -165,7 +177,7 @@ public:
         ss << "uchar";
       else
         ss << vec->basetype->toChars();
-      ss << (int)size;
+      ss << (int)size(vec);
     } else
       ss << t->toChars();
     return ss.str();
