@@ -34,6 +34,8 @@
 #include "ir/irvar.h"
 #include "llvm/ADT/SmallString.h"
 
+using namespace dmd;
+
 //////////////////////////////////////////////////////////////////////////////
 
 class CodegenVisitor : public Visitor {
@@ -107,6 +109,12 @@ public:
       m->accept(this);
     }
 
+    // Objective-C protocols don't have TypeInfo.
+    if (decl->classKind == ClassKind::objc) {
+      gIR->objc.getProtocol(decl);
+      return;
+    }
+
     // Emit TypeInfo.
     IrClass *ir = getIrAggr(decl);
     if (!ir->suppressTypeInfo()) {
@@ -203,6 +211,12 @@ public:
       m->accept(this);
     }
 
+    // Objective-C class structure is initialized by calling getClassRef.
+    if (decl->classKind == ClassKind::objc) {
+      gIR->objc.getClass(decl);
+      return;
+    }
+
     IrClass *ir = getIrAggr(decl);
 
     ir->getInitSymbol(/*define=*/true);
@@ -391,6 +405,11 @@ public:
   void visit(PragmaDeclaration *decl) override {
     const auto &triple = *global.params.targetTriple;
 
+#if LDC_LLVM_VER >= 1800
+    #define endswith ends_with
+    #define startswith starts_with
+#endif
+
     if (decl->ident == Id::lib) {
       assert(!irs->dcomputetarget);
       llvm::StringRef name = getPragmaStringArg(decl);
@@ -464,6 +483,11 @@ public:
       }
     }
     visit(static_cast<AttribDeclaration *>(decl));
+
+#if LDC_LLVM_VER >= 1800
+    #undef endswith
+    #undef startswith
+#endif
   }
 
   //////////////////////////////////////////////////////////////////////////
@@ -471,6 +495,17 @@ public:
   void visit(TypeInfoDeclaration *decl) override {
     llvm_unreachable("Should be emitted from codegen layer only");
   }
+
+  //////////////////////////////////////////////////////////////////////////
+
+  void visit(CAsmDeclaration *ad) override {
+    auto se = ad->code->isStringExp();
+    assert(se);
+
+    DString str = se->peekString();
+    if (str.length)
+      irs->module.appendModuleInlineAsm({str.ptr, str.length});
+  }
 };
 
 //////////////////////////////////////////////////////////////////////////////
