@@ -16,8 +16,8 @@
 class FuncDeclaration;
 
 /// Check whether the frontend knows that the function is already defined
-/// in some other module (see DMD's FuncDeclaration::toObjFile).
-bool alreadyOrWillBeDefined(FuncDeclaration &fdecl);
+/// in some other module (see DMD's `FuncDeclaration_toObjFile()`).
+bool skipCodegen(FuncDeclaration &fdecl);
 
 /// Returns whether `fdecl` should be emitted with externally_available
 /// linkage to make it available for inlining.
