@@ -45,8 +45,12 @@
 #include "llvm/Support/ManagedStatic.h"
 #include "llvm/Target/TargetMachine.h"
 #include "llvm/Transforms/Utils/ModuleUtils.h"
+#include <llvm/IR/Constant.h>
+#include <llvm/Analysis/ConstantFolding.h>
 #include <stack>
 
+using namespace dmd;
+
 llvm::cl::opt<llvm::GlobalVariable::ThreadLocalMode> clThreadModel(
     "fthread-model", llvm::cl::ZeroOrMore, llvm::cl::desc("Thread model"),
     llvm::cl::init(llvm::GlobalVariable::GeneralDynamicTLSModel),
@@ -86,55 +90,45 @@ LLValue *DtoNew(const Loc &loc, Type *newtype) {
   LLConstant *ti = DtoTypeInfoOf(loc, newtype);
   assert(isaPointer(ti));
   // call runtime allocator
-  LLValue *mem = gIR->CreateCallOrInvoke(fn, ti, ".gc_mem");
-  // cast
-  return DtoBitCast(mem, DtoPtrToType(newtype), ".gc_mem");
+  return gIR->CreateCallOrInvoke(fn, ti, ".gc_mem");
 }
 
 void DtoDeleteMemory(const Loc &loc, DValue *ptr) {
   llvm::Function *fn = getRuntimeFunction(loc, gIR->module, "_d_delmemory");
   LLValue *lval = (ptr->isLVal() ? DtoLVal(ptr) : makeLValue(loc, ptr));
-  gIR->CreateCallOrInvoke(
-      fn, DtoBitCast(lval, fn->getFunctionType()->getParamType(0)));
+  gIR->CreateCallOrInvoke(fn, lval);
 }
 
 void DtoDeleteStruct(const Loc &loc, DValue *ptr) {
   llvm::Function *fn = getRuntimeFunction(loc, gIR->module, "_d_delstruct");
   LLValue *lval = (ptr->isLVal() ? DtoLVal(ptr) : makeLValue(loc, ptr));
-  gIR->CreateCallOrInvoke(
-      fn, DtoBitCast(lval, fn->getFunctionType()->getParamType(0)),
-      DtoBitCast(DtoTypeInfoOf(loc, ptr->type->nextOf()),
-                 fn->getFunctionType()->getParamType(1)));
+  gIR->CreateCallOrInvoke(fn, lval, DtoTypeInfoOf(loc, ptr->type->nextOf()));
 }
 
 void DtoDeleteClass(const Loc &loc, DValue *inst) {
   llvm::Function *fn = getRuntimeFunction(loc, gIR->module, "_d_delclass");
   LLValue *lval = (inst->isLVal() ? DtoLVal(inst) : makeLValue(loc, inst));
-  gIR->CreateCallOrInvoke(
-      fn, DtoBitCast(lval, fn->getFunctionType()->getParamType(0)));
+  gIR->CreateCallOrInvoke(fn, lval);
 }
 
 void DtoDeleteInterface(const Loc &loc, DValue *inst) {
   llvm::Function *fn = getRuntimeFunction(loc, gIR->module, "_d_delinterface");
   LLValue *lval = (inst->isLVal() ? DtoLVal(inst) : makeLValue(loc, inst));
-  gIR->CreateCallOrInvoke(
-      fn, DtoBitCast(lval, fn->getFunctionType()->getParamType(0)));
+  gIR->CreateCallOrInvoke(fn, lval);
 }
 
 void DtoDeleteArray(const Loc &loc, DValue *arr) {
   llvm::Function *fn = getRuntimeFunction(loc, gIR->module, "_d_delarray_t");
-  llvm::FunctionType *fty = fn->getFunctionType();
 
   // the TypeInfo argument must be null if the type has no dtor
   Type *elementType = arr->type->nextOf();
   bool hasDtor = (elementType->toBasetype()->ty == TY::Tstruct &&
                   elementType->needsDestruction());
-  LLValue *typeInfo = !hasDtor ? getNullPtr(fty->getParamType(1))
+  LLValue *typeInfo = !hasDtor ? getNullPtr()
                                : DtoTypeInfoOf(loc, elementType);
 
   LLValue *lval = (arr->isLVal() ? DtoLVal(arr) : makeLValue(loc, arr));
-  gIR->CreateCallOrInvoke(fn, DtoBitCast(lval, fty->getParamType(0)),
-                          DtoBitCast(typeInfo, fty->getParamType(1)));
+  gIR->CreateCallOrInvoke(fn, lval, typeInfo);
 }
 
 /******************************************************************************
@@ -245,8 +239,8 @@ LLValue *DtoAllocaDump(LLValue *val, LLType *asType, int alignment,
       (getTypeStoreSize(memType) <= getTypeAllocSize(asMemType) ? asMemType
                                                                 : memType);
   LLValue *mem = DtoRawAlloca(allocaType, alignment, name);
-  DtoStoreZextI8(val, DtoBitCast(mem, memType->getPointerTo()));
-  return DtoBitCast(mem, asMemType->getPointerTo());
+  DtoStoreZextI8(val, mem);
+  return mem;
 }
 
 /******************************************************************************
@@ -296,7 +290,8 @@ void DtoCAssert(Module *M, const Loc &loc, LLValue *msg) {
     args.push_back(line);
     args.push_back(msg);
   } else if (triple.isOSSolaris() || triple.isMusl() ||
-             global.params.isUClibcEnvironment) {
+             global.params.isUClibcEnvironment ||
+             triple.isGNUEnvironment()) {
     const auto irFunc = gIR->func();
     const auto funcName =
         (irFunc && irFunc->decl) ? irFunc->decl->toPrettyChars() : "";
@@ -333,7 +328,7 @@ void DtoCAssert(Module *M, const Loc &loc, LLValue *msg) {
 
 void DtoThrow(const Loc &loc, DValue *e) {
   LLFunction *fn = getRuntimeFunction(loc, gIR->module, "_d_throw_exception");
-  LLValue *arg = DtoBitCast(DtoRVal(e), fn->getFunctionType()->getParamType(0));
+  LLValue *arg = DtoRVal(e);
 
   gIR->CreateCallOrInvoke(fn, arg);
   gIR->ir->CreateUnreachable();
@@ -422,7 +417,6 @@ void DtoAssign(const Loc &loc, DValue *lhs, DValue *rhs, EXP op,
       Logger::cout() << "l : " << *l << '\n';
       Logger::cout() << "r : " << *r << '\n';
     }
-    r = DtoBitCast(r, DtoType(lhs->type));
     DtoStore(r, l);
   } else if (t->iscomplex()) {
     LLValue *dst = DtoLVal(lhs);
@@ -483,7 +477,7 @@ DValue *DtoNullValue(Type *type, Loc loc) {
   // dynamic array
   if (basety == TY::Tarray) {
     LLValue *len = DtoConstSize_t(0);
-    LLValue *ptr = getNullPtr(DtoPtrToType(basetype->nextOf()));
+    LLValue *ptr = getNullPtr();
     return new DSliceValue(type, len, ptr);
   }
   error(loc, "`null` not known for type `%s`", type->toChars());
@@ -506,8 +500,8 @@ DValue *DtoCastInt(const Loc &loc, DValue *val, Type *_to) {
     return new DImValue(_to, rval);
   }
 
-  size_t fromsz = from->size();
-  size_t tosz = to->size();
+  size_t fromsz = size(from);
+  size_t tosz = size(to);
 
   if (to->ty == TY::Tbool) {
     LLValue *zero = LLConstantInt::get(rval->getType(), 0, false);
@@ -589,8 +583,8 @@ DValue *DtoCastFloat(const Loc &loc, DValue *val, Type *to) {
   Type *fromtype = val->type->toBasetype();
   assert(fromtype->isfloating());
 
-  size_t fromsz = fromtype->size();
-  size_t tosz = totype->size();
+  size_t fromsz = size(fromtype);
+  size_t tosz = size(totype);
 
   LLValue *rval;
 
@@ -652,7 +646,7 @@ DValue *DtoCastVector(const Loc &loc, DValue *val, Type *to) {
       LLValue *vector = DtoLVal(val);
       IF_LOG Logger::cout() << "src: " << *vector << " to type: " << *tolltype
                             << " (casting address)\n";
-      return new DLValue(to, DtoBitCast(vector, getPtrToType(tolltype)));
+      return new DLValue(to, vector);
     }
 
     LLValue *vector = DtoRVal(val);
@@ -661,7 +655,7 @@ DValue *DtoCastVector(const Loc &loc, DValue *val, Type *to) {
     LLValue *array = DtoAllocaDump(vector, tolltype, DtoAlignment(val->type));
     return new DLValue(to, array);
   }
-  if (totype->ty == TY::Tvector && to->size() == val->type->size()) {
+  if (totype->ty == TY::Tvector && size(to) == size(val->type)) {
     return new DImValue(to, DtoBitCast(DtoRVal(val), tolltype));
   }
   error(loc, "invalid cast from `%s` to `%s`", val->type->toChars(),
@@ -676,9 +670,7 @@ DValue *DtoCastStruct(const Loc &loc, DValue *val, Type *to) {
     // allows for identical layouts (opCast() and so on have been lowered
     // earlier by the frontend).
     llvm::Value *lval = DtoLVal(val);
-    llvm::Value *result = DtoBitCast(lval, DtoType(to)->getPointerTo(),
-                                     lval->getName() + ".repaint");
-    return new DLValue(to, result);
+    return new DLValue(to, lval);
   }
 
   error(loc, "Internal Compiler Error: Invalid struct cast from `%s` to `%s`",
@@ -701,8 +693,7 @@ DValue *DtoCast(const Loc &loc, DValue *val, Type *to) {
     // implemented as structs.
     if (totype->ty == TY::Tpointer) {
       IF_LOG Logger::println("Casting AA to pointer.");
-      LLValue *rval = DtoBitCast(DtoRVal(val), DtoType(to));
-      return new DImValue(to, rval);
+      return new DImValue(to, DtoRVal(val));
     }
     if (totype->ty == TY::Tbool) {
       IF_LOG Logger::println("Casting AA to bool.");
@@ -767,20 +758,16 @@ DValue *DtoPaintType(const Loc &loc, DValue *val, Type *to) {
   Type *tb = to->toBasetype();
 
   if (val->isLVal()) {
-    auto ptr = DtoBitCast(DtoLVal(val), DtoPtrToType(tb));
-    return new DLValue(to, ptr);
+    return new DLValue(to, DtoLVal(val));
   }
 
   if (auto slice = val->isSlice()) {
     if (tb->ty == TY::Tarray) {
-      auto ptr = DtoBitCast(slice->getPtr(), DtoPtrToType(tb->nextOf()));
-      return new DSliceValue(to, slice->getLength(), ptr);
+      return new DSliceValue(to, slice->getLength(), slice->getPtr());
     }
   } else if (auto func = val->isFunc()) {
     if (tb->ty == TY::Tdelegate) {
-      auto funcptr =
-          DtoBitCast(DtoRVal(func), DtoType(tb)->getContainedType(1));
-      return new DFuncValue(to, func->func, funcptr, func->vthis);
+      return new DFuncValue(to, func->func, DtoRVal(func), func->vthis);
     }
   } else { // generic rvalue
     LLValue *rval = DtoRVal(val);
@@ -790,12 +777,11 @@ DValue *DtoPaintType(const Loc &loc, DValue *val, Type *to) {
       return new DImValue(to, rval);
     }
     if (rval->getType()->isPointerTy() && tll->isPointerTy()) {
-      return new DImValue(to, DtoBitCast(rval, tll));
+      return new DImValue(to, rval);
     }
     if (from->ty == TY::Tdelegate && tb->ty == TY::Tdelegate) {
       LLValue *context = gIR->ir->CreateExtractValue(rval, 0, ".context");
       LLValue *funcptr = gIR->ir->CreateExtractValue(rval, 1, ".funcptr");
-      funcptr = DtoBitCast(funcptr, tll->getContainedType(1));
       return new DImValue(to, DtoAggrPair(context, funcptr));
     }
   }
@@ -907,13 +893,13 @@ void DtoVarDeclaration(VarDeclaration *vd) {
     // already been done
     IrLocal *irLocal = getIrLocal(vd, true);
 
-    Type *type = isSpecialRefVar(vd) ? vd->type->pointerTo() : vd->type;
+    Type *type = isSpecialRefVar(vd) ? pointerTo(vd->type) : vd->type;
 
     llvm::Value *allocainst;
     bool isRealAlloca = false;
     LLType *lltype = DtoType(type); // void for noreturn
     if (lltype->isVoidTy() || gDataLayout->getTypeSizeInBits(lltype) == 0) {
-      allocainst = llvm::ConstantPointerNull::get(getPtrToType(lltype));
+      allocainst = getNullPtr();
     } else if (type != vd->type) {
       allocainst = DtoAlloca(type, vd->toChars());
       isRealAlloca = true;
@@ -931,7 +917,7 @@ void DtoVarDeclaration(VarDeclaration *vd) {
     if (isRealAlloca) {
       // The lifetime of a stack variable starts from the point it is declared
       gIR->funcGen().localVariableLifetimeAnnotator.addLocalVariable(
-          allocainst, DtoConstUlong(type->size()));
+          allocainst, DtoConstUlong(size(type)));
     }
   }
 
@@ -1139,7 +1125,6 @@ LLConstant *DtoConstExpInit(const Loc &loc, Type *targetType, Expression *exp) {
 
   LLType *llType = val->getType();
   LLType *targetLLType = DtoMemType(baseTargetType);
-
   // shortcut for zeros
   if (val->isNullValue())
     return llvm::Constant::getNullValue(targetLLType);
@@ -1147,7 +1132,11 @@ LLConstant *DtoConstExpInit(const Loc &loc, Type *targetType, Expression *exp) {
   // extend i1 to i8
   if (llType->isIntegerTy(1)) {
     llType = LLType::getInt8Ty(gIR->context());
+#if LDC_LLVM_VER < 1800
     val = llvm::ConstantExpr::getZExt(val, llType);
+#else
+    val = llvm::ConstantFoldCastOperand(llvm::Instruction::ZExt, val, llType, *gDataLayout);
+#endif
   }
 
   if (llType == targetLLType)
@@ -1156,9 +1145,9 @@ LLConstant *DtoConstExpInit(const Loc &loc, Type *targetType, Expression *exp) {
   if (baseTargetType->ty == TY::Tsarray) {
     Logger::println("Building constant array initializer from scalar.");
 
-    assert(baseValType->size() > 0);
-    const auto numTotalVals = baseTargetType->size() / baseValType->size();
-    assert(baseTargetType->size() % baseValType->size() == 0);
+    assert(size(baseValType) > 0);
+    const auto numTotalVals = size(baseTargetType) / size(baseValType);
+    assert(size(baseTargetType) % size(baseValType) == 0);
 
     // may be a multi-dimensional array init, e.g., `char[2][3] x = 0xff`
     baseValType = stripModifiers(baseValType);
@@ -1191,11 +1180,7 @@ LLConstant *DtoConstExpInit(const Loc &loc, Type *targetType, Expression *exp) {
     assert(tv->basetype->ty == TY::Tsarray);
     dinteger_t elemCount =
         static_cast<TypeSArray *>(tv->basetype)->dim->toInteger();
-#if LDC_LLVM_VER >= 1200
     const auto elementCount = llvm::ElementCount::getFixed(elemCount);
-#else
-    const auto elementCount = llvm::ElementCount(elemCount, false);
-#endif
     return llvm::ConstantVector::getSplat(elementCount, val);
   }
 
@@ -1211,7 +1196,11 @@ LLConstant *DtoConstExpInit(const Loc &loc, Type *targetType, Expression *exp) {
            "On initializer integer type mismatch, the target should be wider "
            "than the source.");
 
+#if LDC_LLVM_VER < 1800
     return llvm::ConstantExpr::getZExtOrBitCast(val, target);
+#else
+    return llvm::ConstantFoldCastOperand(llvm::Instruction::ZExt, val, target, *gDataLayout);
+#endif
   }
 
   Logger::println("Unhandled type mismatch, giving up.");
@@ -1220,16 +1209,12 @@ LLConstant *DtoConstExpInit(const Loc &loc, Type *targetType, Expression *exp) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-LLConstant *DtoTypeInfoOf(const Loc &loc, Type *type, bool base) {
-  IF_LOG Logger::println("DtoTypeInfoOf(type = '%s', base='%d')",
-                         type->toChars(), base);
+LLConstant *DtoTypeInfoOf(const Loc &loc, Type *type) {
+  IF_LOG Logger::println("DtoTypeInfoOf(type = '%s')", type->toChars());
   LOG_SCOPE
 
   auto tidecl = getOrCreateTypeInfoDeclaration(loc, type);
   auto tiglobal = DtoResolveTypeInfo(tidecl);
-  if (base) {
-    return llvm::ConstantExpr::getBitCast(tiglobal, DtoType(getTypeInfoType()));
-  }
   return tiglobal;
 }
 
@@ -1240,6 +1225,8 @@ static char *DtoOverloadedIntrinsicName(TemplateInstance *ti,
   IF_LOG Logger::println("DtoOverloadedIntrinsicName");
   LOG_SCOPE;
 
+  assert(td->intrinsicName);
+
   IF_LOG {
     Logger::println("template instance: %s", ti->toChars());
     Logger::println("template declaration: %s", td->toChars());
@@ -1302,7 +1289,7 @@ static char *DtoOverloadedIntrinsicName(TemplateInstance *ti,
 
   IF_LOG Logger::println("final intrinsic name: %s", name.c_str());
 
-  return strdup(name.c_str());
+  return mem.xstrdup(name.c_str());
 }
 
 /// For D frontend
@@ -1310,11 +1297,9 @@ static char *DtoOverloadedIntrinsicName(TemplateInstance *ti,
 void DtoSetFuncDeclIntrinsicName(TemplateInstance *ti, TemplateDeclaration *td,
                                  FuncDeclaration *fd) {
   if (fd->llvmInternal == LLVMintrinsic) {
-    fd->intrinsicName = DtoOverloadedIntrinsicName(ti, td);
-    const auto cstr = fd->intrinsicName;
-    fd->mangleOverride = {cstr ? strlen(cstr) : 0, cstr};
-  } else {
-    fd->intrinsicName = td->intrinsicName ? strdup(td->intrinsicName) : nullptr;
+    const auto cstr = DtoOverloadedIntrinsicName(ti, td);
+    assert(cstr);
+    fd->mangleOverride = {strlen(cstr), cstr};
   }
 }
 
@@ -1326,9 +1311,9 @@ Type *stripModifiers(Type *type, bool transitive) {
   }
 
   if (transitive) {
-    return type->unqualify(MODimmutable | MODconst | MODwild);
+    return unqualify(type, MODimmutable | MODconst | MODwild);
   }
-  return type->castMod(0);
+  return castMod(type, 0);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1501,11 +1486,7 @@ DValue *DtoSymbolAddress(const Loc &loc, Type *type, Declaration *decl) {
     // typeinfo
     if (TypeInfoDeclaration *tid = vd->isTypeInfoDeclaration()) {
       Logger::println("TypeInfoDeclaration");
-      LLType *vartype = DtoType(type);
       LLValue *m = DtoResolveTypeInfo(tid);
-      if (m->getType() != getPtrToType(vartype)) {
-        m = gIR->ir->CreateBitCast(m, vartype);
-      }
       return new DImValue(type, m);
     }
     // special vtbl symbol, used by LDC as alias to the actual vtbl (with
@@ -1513,8 +1494,7 @@ DValue *DtoSymbolAddress(const Loc &loc, Type *type, Declaration *decl) {
     if (vd->isClassMember() && vd == vd->isClassMember()->vtblsym) {
       Logger::println("vtbl symbol");
       auto cd = vd->isClassMember();
-      return new DLValue(
-          type, DtoBitCast(getIrAggr(cd)->getVtblSymbol(), DtoPtrToType(type)));
+      return new DLValue(type, getIrAggr(cd)->getVtblSymbol());
     }
     // nested variable
     if (vd->nestedrefs.length) {
@@ -1539,7 +1519,7 @@ DValue *DtoSymbolAddress(const Loc &loc, Type *type, Declaration *decl) {
       assert(!isSpecialRefVar(vd) && "Code not expected to handle special "
                                      "ref vars, although it can easily be "
                                      "made to.");
-      return new DLValue(type, DtoBitCast(getIrValue(vd), DtoPtrToType(type)));
+      return new DLValue(type, getIrValue(vd));
     }
     Logger::println("a normal variable");
 
@@ -1574,9 +1554,8 @@ DValue *DtoSymbolAddress(const Loc &loc, Type *type, Declaration *decl) {
       fatal();
     }
     DtoResolveFunction(fdecl);
-    const auto llValue =
-        fdecl->llvmInternal != LLVMva_arg ? DtoCallee(fdecl) : nullptr;
-    return new DFuncValue(fdecl, llValue);
+    assert(!DtoIsMagicIntrinsic(fdecl));
+    return new DFuncValue(fdecl, DtoCallee(fdecl));
   }
 
   if (SymbolDeclaration *sdecl = decl->isSymbolDeclaration()) {
@@ -1591,10 +1570,9 @@ DValue *DtoSymbolAddress(const Loc &loc, Type *type, Declaration *decl) {
     if (tb->ty != TY::Tstruct) {
       assert(tb->ty == TY::Tarray && tb->nextOf()->ty == TY::Tvoid);
       const auto size = DtoConstSize_t(ad->structsize);
-      llvm::Constant *ptr =
-          sd && sd->zeroInit()
-              ? getNullValue(getVoidPtrType())
-              : DtoBitCast(getIrAggr(ad)->getInitSymbol(), getVoidPtrType());
+      LLConstant *ptr = sd && sd->zeroInit()
+                            ? static_cast<LLConstant *>(getNullPtr())
+                            : getIrAggr(ad)->getInitSymbol();
       return new DSliceValue(type, size, ptr);
     }
 
@@ -1605,7 +1583,7 @@ DValue *DtoSymbolAddress(const Loc &loc, Type *type, Declaration *decl) {
     }
 
     LLValue *initsym = getIrAggr(sd)->getInitSymbol();
-    return new DLValue(type, DtoBitCast(initsym, DtoPtrToType(sd->type)));
+    return new DLValue(type, initsym);
   }
 
   llvm_unreachable("Unimplemented VarExp type");
@@ -1644,7 +1622,7 @@ llvm::Constant *DtoConstSymbolAddress(const Loc &loc, Declaration *decl) {
 
 llvm::Constant *buildStringLiteralConstant(StringExp *se,
                                            uint64_t bufferLength) {
-  const auto stringLength = se->numberOfCodeUnits();
+  const auto stringLength = se->len;
   assert(bufferLength >= stringLength);
 
   if (se->sz == 1 && bufferLength <= stringLength + 1) {
@@ -1662,7 +1640,7 @@ llvm::Constant *buildStringLiteralConstant(StringExp *se,
   std::vector<LLConstant *> vals;
   vals.reserve(bufferLength);
   for (uint64_t i = 0; i < stringLength; ++i) {
-    vals.push_back(LLConstantInt::get(ct, se->getCodeUnit(i), false));
+    vals.push_back(LLConstantInt::get(ct, se->getIndex(i), false));
   }
   const auto nullChar = LLConstantInt::get(ct, 0, false);
   for (uint64_t i = stringLength; i < bufferLength; ++i) {
@@ -1863,6 +1841,18 @@ DLValue *DtoIndexAggregate(LLValue *src, AggregateDeclaration *ad,
   // ourselves, DtoType below would be enough.
   DtoResolveDsymbol(ad);
 
+  if (ad->classKind == ClassKind::objc) {
+    auto tHandle = getI32Type();
+    auto tOffset = DtoLoad(tHandle, gIR->objc.getIvar(vd)->offset);
+
+    // Offset is now stored in tOffset.
+    LLValue *ptr = src;
+    ptr = DtoBitCast(ptr, getOpaquePtrType());
+    ptr = DtoGEP1(getI8Type(), ptr, tOffset);
+
+    return new DLValue(vd->type, ptr);
+  }
+
   // Look up field to index or offset to apply.
   auto irTypeAggr = getIrType(ad->type)->isAggr();
   assert(irTypeAggr);
@@ -1872,9 +1862,8 @@ DLValue *DtoIndexAggregate(LLValue *src, AggregateDeclaration *ad,
   LLValue *ptr = src;
   LLType * ty = nullptr;
   if (!isFieldIdx) {
-    // Cast to void* to apply byte-wise offset from object start.
-    ptr = DtoBitCast(ptr, getVoidPtrType());
-    ptr = DtoGEP1(llvm::Type::getInt8Ty(gIR->context()), ptr, off);
+    // apply byte-wise offset from object start
+    ptr = DtoGEP1(getI8Type(), ptr, off);
     ty = DtoType(vd->type);
   } else {
     if (ad->structsize == 0) { // can happen for extern(C) structs
@@ -1888,15 +1877,11 @@ DLValue *DtoIndexAggregate(LLValue *src, AggregateDeclaration *ad,
       } else {
         st = irTypeAggr->getLLType();
       }
-      ptr = DtoBitCast(ptr, st->getPointerTo());
       ptr = DtoGEP(st, ptr, 0, off);
       ty = isaStruct(st)->getElementType(off);
     }
   }
 
-  // Cast the (possibly void*) pointer to the canonical variable type.
-  ptr = DtoBitCast(ptr, DtoPtrToType(vd->type));
-
   IF_LOG Logger::cout() << "Pointer: " << *ptr << '\n';
   if (auto p = isaPointer(ty)) {
     if (p->getAddressSpace())
@@ -1921,31 +1906,9 @@ DValue *makeVarDValue(Type *type, VarDeclaration *vd, llvm::Value *storage) {
     val = getIrValue(vd);
   }
 
-  // We might need to cast.
-  llvm::Type *expectedType = DtoPtrToType(type);
-  const bool isSpecialRef = isSpecialRefVar(vd);
-  if (isSpecialRef)
-    expectedType = expectedType->getPointerTo();
+  assert(val->getType()->isPointerTy());
 
-  if (val->getType() != expectedType) {
-#if LDC_LLVM_VER < 1500
-    // The type of globals is determined by their initializer, and the front-end
-    // may inject implicit casts for class references and static arrays.
-    assert(vd->isDataseg() || (vd->storage_class & STCextern) ||
-           type->toBasetype()->ty == TY::Tclass ||
-           type->toBasetype()->ty == TY::Tsarray);
-    llvm::Type *pointeeType = val->getType()->getPointerElementType();
-    if (isSpecialRef)
-      pointeeType = pointeeType->getPointerElementType();
-    // Make sure that the type sizes fit - '==' instead of '<=' should probably
-    // work as well.
-    assert(getTypeStoreSize(DtoType(type)) <= getTypeStoreSize(pointeeType) &&
-           "LValue type mismatch, encountered type too small.");
-#endif
-    val = DtoBitCast(val, expectedType);
-  }
-
-  if (isSpecialRef)
+  if (isSpecialRefVar(vd))
     return new DSpecialRefValue(type, val);
 
   return new DLValue(type, val);
