@@ -25,6 +25,8 @@
 #include "llvm/IR/InlineAsm.h"
 #include <cassert>
 
+using namespace dmd;
+
 ////////////////////////////////////////////////////////////////////////////////
 // FIXME: Integrate these functions
 void AsmStatement_toNakedIR(InlineAsmStatement *stmt, IRState *irs);
@@ -270,7 +272,7 @@ void emitABIReturnAsmStmt(IRAsmBlock *asmblock, const Loc &loc,
   if (triple.getArch() == llvm::Triple::x86) {
     if (rt->isintegral() || rt->ty == TY::Tpointer || rt->ty == TY::Tclass ||
         rt->ty == TY::Taarray) {
-      if (rt->size() == 8) {
+      if (size(rt) == 8) {
         as->out.c = "=A,";
       } else {
         as->out.c = "={ax},";
@@ -480,7 +482,7 @@ DValue *DtoInlineAsmExpr(const Loc &loc, FuncDeclaration *fd,
     auto lvalue = sretPointer;
     if (!lvalue)
       lvalue = DtoAlloca(returnType, ".__asm_tuple_ret");
-    DtoStore(rv, DtoBitCast(lvalue, getPtrToType(irReturnType)));
+    DtoStore(rv, lvalue);
     return new DLValue(returnType, lvalue);
   }
 
@@ -505,20 +507,11 @@ llvm::CallInst *DtoInlineAsmExpr(const Loc &loc, llvm::StringRef code,
   llvm::FunctionType *FT =
       llvm::FunctionType::get(returnType, operandTypes, false);
 
-#if LDC_LLVM_VER < 1500
-  // make sure the constraints are valid
-  if (!llvm::InlineAsm::Verify(FT, constraints)) {
-      error(loc, "inline asm constraints are invalid");
-      fatal();
-  }
-#else
   if (auto err = llvm::InlineAsm::verify(FT, constraints)) {
     error(loc, "inline asm constraints are invalid");
     llvm::errs() << err;
     fatal();
   }
-#endif
-
 
   // build asm call
   bool sideeffect = true;
