@@ -14,7 +14,9 @@
 #include "dmd/id.h"
 #include "dmd/mtype.h"
 #include "dmd/target.h"
+#include "dmd/template.h"
 #include "gen/abi/abi.h"
+#include "gen/arrays.h"
 #include "gen/classes.h"
 #include "gen/dvalue.h"
 #include "gen/funcgenstate.h"
@@ -31,6 +33,9 @@
 #include "ir/irfunction.h"
 #include "ir/irtype.h"
 #include "llvm/IR/LLVMContext.h"
+#include <llvm/IR/DerivedTypes.h>
+
+using namespace dmd;
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -97,17 +102,9 @@ static void addExplicitArguments(std::vector<LLValue *> &args, AttrSet &attrs,
     Type *argType = argexps[i]->type;
     bool passByVal = gABI->passByVal(irFty.type, argType);
 
-#if LDC_LLVM_VER >= 1400
     llvm::AttrBuilder initialAttrs(getGlobalContext());
-#else
-    llvm::AttrBuilder initialAttrs;
-#endif
     if (passByVal) {
-#if LDC_LLVM_VER >= 1200
       initialAttrs.addByValAttr(DtoType(argType));
-#else
-      initialAttrs.addAttribute(LLAttribute::ByVal);
-#endif
       if (auto alignment = DtoAlignment(argType))
         initialAttrs.addAlignmentAttr(alignment);
     } else {
@@ -241,35 +238,11 @@ static LLValue *getTypeinfoArrayArgumentForDVarArg(Expressions *argexps,
   LLConstant *tiinits = LLConstantArray::get(typeinfoarraytype, vtypeinfos);
   typeinfomem->setInitializer(tiinits);
 
-  // put data in d-array
-  LLConstant *pinits[] = {
-      DtoConstSize_t(numVariadicArgs),
-      llvm::ConstantExpr::getBitCast(typeinfomem, getPtrToType(typeinfotype))};
-  LLType *tiarrty = DtoType(getTypeInfoType()->arrayOf());
-  tiinits = LLConstantStruct::get(isaStruct(tiarrty),
-                                  llvm::ArrayRef<LLConstant *>(pinits));
-  LLValue *typeinfoarrayparam = new llvm::GlobalVariable(
-      gIR->module, tiarrty, true, llvm::GlobalValue::InternalLinkage, tiinits,
-      "._arguments.array");
-
-  return DtoLoad(tiarrty, typeinfoarrayparam);
+  return DtoConstSlice(DtoConstSize_t(numVariadicArgs), typeinfomem);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
-static LLType *getPtrToAtomicType(LLType *type) {
-  switch (const size_t N = getTypeBitSize(type)) {
-  case 8:
-  case 16:
-  case 32:
-  case 64:
-  case 128:
-    return LLType::getIntNPtrTy(gIR->context(), static_cast<unsigned>(N));
-  default:
-    return nullptr;
-  }
-}
-
 static LLType *getAtomicType(LLType *type) {
   switch (const size_t N = getTypeBitSize(type)) {
   case 8:
@@ -300,7 +273,7 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
       gABI->vaCopy(ap, &argptr);
     } else {
       LLValue *llAp = gABI->prepareVaStart(ap);
-      p->ir->CreateCall(GET_INTRINSIC_DECL(vastart), llAp, "");
+      p->ir->CreateCall(GET_INTRINSIC_DECL(vastart, llAp->getType()), llAp, "");
     }
     result = nullptr;
     return true;
@@ -348,7 +321,7 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
     DLValue *ap = toElem((*e->arguments)[0])->isLVal(); // va_list
     assert(ap);
     LLValue *llAp = gABI->prepareVaArg(ap);
-    p->ir->CreateCall(GET_INTRINSIC_DECL(vaend), llAp);
+    p->ir->CreateCall(GET_INTRINSIC_DECL(vaend, llAp->getType()), llAp);
     result = nullptr;
     return true;
   }
@@ -382,7 +355,12 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
     if (e->arguments->length == 2) {
       scope = static_cast<llvm::SyncScope::ID>((*e->arguments)[1]->toInteger());
     }
-    p->ir->CreateFence(atomicOrdering, scope);
+    // orderings below acquire are invalid; like clang, don't emit any
+    // instruction in those cases
+    if (static_cast<int>(atomicOrdering) >
+        static_cast<int>(llvm::AtomicOrdering::Monotonic)) {
+      p->ir->CreateFence(atomicOrdering, scope);
+    }
     return true;
   }
 
@@ -403,11 +381,9 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
     LLValue *val = nullptr;
     if (pointeeType->isIntegerTy()) {
       val = DtoRVal(dval);
-    } else if (auto intPtrType = getPtrToAtomicType(pointeeType)) {
-      LLType *atype = getAtomicType(pointeeType);
-      ptr = DtoBitCast(ptr, intPtrType);
+    } else if (auto atype = getAtomicType(pointeeType)) {
       auto lval = makeLValue(exp1->loc, dval);
-      val = DtoLoad(atype, DtoBitCast(lval, intPtrType));
+      val = DtoLoad(atype, lval);
     } else {
       error(e->loc,
           "atomic store only supports types of size 1/2/4/8/16 bytes, not `%s`",
@@ -439,9 +415,8 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
     Type *retType = exp->type->nextOf();
 
     if (!pointeeType->isIntegerTy()) {
-      if (auto intPtrType = getPtrToAtomicType(pointeeType)) {
-        ptr = DtoBitCast(ptr, intPtrType);
-        loadedType = getAtomicType(pointeeType);
+      if (auto atype = getAtomicType(pointeeType)) {
+        loadedType = atype;
       } else {
         error(e->loc,
               "atomic load only supports types of size 1/2/4/8/16 bytes, "
@@ -495,13 +470,11 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
     if (pointeeType->isIntegerTy()) {
       cmp = DtoRVal(dcmp);
       val = DtoRVal(dval);
-    } else if (auto intPtrType = getPtrToAtomicType(pointeeType)) {
-      LLType *atype = getAtomicType(pointeeType);
-      ptr = DtoBitCast(ptr, intPtrType);
+    } else if (auto atype = getAtomicType(pointeeType)) {
       auto cmpLVal = makeLValue(exp2->loc, dcmp);
-      cmp = DtoLoad(atype, DtoBitCast(cmpLVal, intPtrType));
+      cmp = DtoLoad(atype, cmpLVal);
       auto lval = makeLValue(exp3->loc, dval);
-      val = DtoLoad(atype, DtoBitCast(lval, intPtrType));
+      val = DtoLoad(atype, lval);
     } else {
       error(e->loc,
             "`cmpxchg` only supports types of size 1/2/4/8/16 bytes, not `%s`",
@@ -510,11 +483,9 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
     }
 
     auto ret =
-      p->ir->CreateAtomicCmpXchg(ptr, cmp, val,
-#if LDC_LLVM_VER >= 1300
-                                 llvm::MaybeAlign(), // default alignment
-#endif
-                                 successOrdering, failureOrdering);
+        p->ir->CreateAtomicCmpXchg(ptr, cmp, val,
+                                   llvm::MaybeAlign(), // default alignment
+                                   successOrdering, failureOrdering);
     ret->setWeak(isWeak);
 
     // we return a struct; allocate on stack and store to both fields manually
@@ -522,8 +493,7 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
     // because of i1)
     auto mem = DtoAlloca(e->type);
     llvm::Type* memty = DtoType(e->type);
-    DtoStore(p->ir->CreateExtractValue(ret, 0),
-             DtoBitCast(DtoGEP(memty, mem, 0u, 0), ptr->getType()));
+    DtoStore(p->ir->CreateExtractValue(ret, 0), DtoGEP(memty, mem, 0u, 0));
     DtoStoreZextI8(p->ir->CreateExtractValue(ret, 1), DtoGEP(memty, mem, 0, 1));
 
     result = new DLValue(e->type, mem);
@@ -537,18 +507,21 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
       fatal();
     }
 
-    assert(fndecl->intrinsicName);
+    TemplateInstance *ti = fndecl->parent->isTemplateInstance();
+    assert(ti);
+    const char *opString = ti->tempdecl->isTemplateDeclaration()->intrinsicName;
+    assert(opString);
+
     static const char *ops[] = {"xchg", "add", "sub", "and",  "nand", "or",
                                 "xor",  "max", "min", "umax", "umin", nullptr};
 
     int op = 0;
     for (;; ++op) {
       if (ops[op] == nullptr) {
-        error(e->loc, "unknown `atomicrmw` operation `%s`",
-              fndecl->intrinsicName);
+        error(e->loc, "unknown `atomicrmw` operation `%s`", opString);
         fatal();
       }
-      if (strcmp(fndecl->intrinsicName, ops[op]) == 0) {
+      if (strcmp(opString, ops[op]) == 0) {
         break;
       }
     }
@@ -560,9 +533,7 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
     LLValue *val = DtoRVal(exp2);
     LLValue *ret =
         p->ir->CreateAtomicRMW(llvm::AtomicRMWInst::BinOp(op), ptr, val,
-#if LDC_LLVM_VER >= 1300
                                llvm::MaybeAlign(), // default alignment
-#endif
                                llvm::AtomicOrdering(atomicOrdering));
     result = new DImValue(exp2->type, ret);
     return true;
@@ -586,8 +557,9 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
     unsigned bitmask = DtoSize_t()->getBitWidth() - 1;
     assert(bitmask == 31 || bitmask == 63);
     // auto q = cast(size_t*)ptr + (bitnum >> (64bit ? 6 : 5));
-    LLValue *q = DtoBitCast(ptr, DtoSize_t()->getPointerTo());
-    q = DtoGEP1(DtoSize_t(), q, p->ir->CreateLShr(bitnum, bitmask == 63 ? 6 : 5), "bitop.q");
+    LLValue *q =
+        DtoGEP1(DtoSize_t(), ptr,
+                p->ir->CreateLShr(bitnum, bitmask == 63 ? 6 : 5), "bitop.q");
 
     // auto mask = 1 << (bitnum & bitmask);
     LLValue *mask =
@@ -674,13 +646,13 @@ public:
         tf(DtoTypeFunction(fnval)),
         llArgTypesBegin(llCalleeType->param_begin()) {}
 
-  void addImplicitArgs() {
+  void addImplicitArgs(bool directcall) {
     if (gABI->passThisBeforeSret(tf)) {
-      addContext();
+      addContext(directcall);
       addSret();
     } else {
       addSret();
-      addContext();
+      addContext(directcall);
     }
 
     addArguments();
@@ -730,16 +702,17 @@ private:
   }
 
   // Adds an optional context/this pointer argument and sets hasContext.
-  void addContext() {
+  void addContext(bool directcall) {
     const bool thiscall = irFty.arg_this;
     const bool nestedcall = irFty.arg_nest;
+    const bool objccall = irFty.type->linkage == LINK::objc;
 
-    hasContext = thiscall || nestedcall || isDelegateCall;
+    hasContext = thiscall || nestedcall || isDelegateCall || objccall;
     if (!hasContext)
       return;
 
     size_t index = args.size();
-    LLType *llArgType = *(llArgTypesBegin + index);
+    auto argtype = *(llArgTypesBegin + index);
 
     if (dfnval && (dfnval->func->ident == Id::ensure ||
                    dfnval->func->ident == Id::require)) {
@@ -759,30 +732,53 @@ private:
           }
         }
       }
-      LLValue *contextptr = DtoBitCast(thisptrLval, getVoidPtrType());
-      args.push_back(contextptr);
+      args.push_back(thisptrLval);
     } else if (thiscall && dfnval && dfnval->vthis) {
-      // ... or a normal 'this' argument
-      LLValue *thisarg = DtoBitCast(dfnval->vthis, llArgType);
-      args.push_back(thisarg);
+      
+      if (objccall && directcall) {
+
+        // ... or a Objective-c direct call argument
+        if (auto func = dfnval->func->isFuncDeclaration()) {
+          if (auto klass = func->isThis()->isClassDeclaration()) {
+
+            // Create obj_super struct with (this, <class ref>)
+            auto obj_super = DtoAggrPair(
+              DtoBitCast(dfnval->vthis, argtype),
+              gIR->objc.deref(klass, getOpaquePtrType()),
+              "super"
+            );
+
+
+            // Allocate and store obj_super struct into a new variable.
+            auto clsaddr = DtoRawAlloca(obj_super->getType(), 16, "super");
+            DtoStore(obj_super, clsaddr);
+
+            args.push_back(clsaddr);
+          }
+        }
+      } else {
+
+        // ... or a normal 'this' argument
+        args.push_back(DtoBitCast(dfnval->vthis, argtype));
+      }
     } else if (isDelegateCall) {
       // ... or a delegate context arg
       LLValue *ctxarg;
       if (fnval->isLVal()) {
-        ctxarg = DtoLoad(getVoidPtrType(), DtoGEP(DtoType(fnval->type), DtoLVal(fnval), 0u, 0), ".ptr");
+        ctxarg = DtoLoad(getOpaquePtrType(),
+                         DtoGEP(DtoType(fnval->type), DtoLVal(fnval), 0u, 0),
+                         ".ptr");
       } else {
         ctxarg = gIR->ir->CreateExtractValue(DtoRVal(fnval), 0, ".ptr");
       }
-      ctxarg = DtoBitCast(ctxarg, llArgType);
       args.push_back(ctxarg);
     } else if (nestedcall) {
       // ... or a nested function context arg
       if (dfnval) {
         LLValue *contextptr = DtoNestedContext(loc, dfnval->func);
-        contextptr = DtoBitCast(contextptr, getVoidPtrType());
         args.push_back(contextptr);
       } else {
-        args.push_back(llvm::UndefValue::get(getVoidPtrType()));
+        args.push_back(llvm::UndefValue::get(getOpaquePtrType()));
       }
     } else {
       error(loc, "Context argument required but none given");
@@ -798,11 +794,9 @@ private:
 
     if (irFty.arg_objcSelector) {
       assert(dfnval);
-      const auto selector = dfnval->func->objc.selector;
-      assert(selector);
-      LLGlobalVariable *selptr = gIR->objc.getMethVarRef(*selector);
-      args.push_back(DtoBitCast(DtoLoad(selptr->getValueType(), selptr),
-                                getVoidPtrType()));
+
+      auto methodptr = gIR->objc.getMethod(dfnval->func)->selector;
+      args.push_back(DtoLoad(methodptr->getType(), methodptr));
     }
   }
 
@@ -845,7 +839,7 @@ static LLValue *DtoCallableValue(DValue *fn) {
 
 // FIXME: this function is a mess !
 DValue *DtoCallFunction(const Loc &loc, Type *resulttype, DValue *fnval,
-                        Expressions *arguments, LLValue *sretPointer) {
+                        Expressions *arguments, LLValue *sretPointer, bool directcall) {
   IF_LOG Logger::println("DtoCallFunction()");
   LOG_SCOPE
 
@@ -890,7 +884,7 @@ DValue *DtoCallFunction(const Loc &loc, Type *resulttype, DValue *fnval,
   // handle implicit arguments (sret, context/this, _arguments)
   ImplicitArgumentsBuilder iab(args, attrs, loc, fnval, callableTy, arguments,
                                resulttype, sretPointer);
-  iab.addImplicitArgs();
+  iab.addImplicitArgs(directcall);
 
   // handle explicit arguments
 
@@ -912,9 +906,10 @@ DValue *DtoCallFunction(const Loc &loc, Type *resulttype, DValue *fnval,
   }
 
   if (irFty.arg_objcSelector) {
+
     // Use runtime msgSend function bitcasted as original call
-    const char *msgSend = gABI->objcMsgSendFunc(resulttype, irFty);
-    LLType *t = callable->getType();
+    const char *msgSend = gABI->objcMsgSendFunc(resulttype, irFty, directcall);
+    auto t = callable->getType();
     callable = getRuntimeFunction(loc, gIR->module, msgSend);
     callable = DtoBitCast(callable, t);
   }
@@ -958,7 +953,7 @@ DValue *DtoCallFunction(const Loc &loc, Type *resulttype, DValue *fnval,
     switch (rbase->ty) {
     case TY::Tarray:
       if (tf->isref()) {
-        retllval = DtoBitCast(retllval, DtoType(rbase->pointerTo()));
+        retllval = DtoBitCast(retllval, DtoType(pointerTo(rbase)));
       } else {
         retllval = DtoSlicePaint(retllval, DtoType(rbase));
       }
@@ -966,9 +961,7 @@ DValue *DtoCallFunction(const Loc &loc, Type *resulttype, DValue *fnval,
 
     case TY::Tsarray:
       if (nextbase->ty == TY::Tvector && !tf->isref()) {
-        if (retValIsLVal) {
-          retllval = DtoBitCast(retllval, DtoType(rbase->pointerTo()));
-        } else {
+        if (!retValIsLVal) {
           // static arrays need to be dumped to memory; use vector alignment
           retllval =
               DtoAllocaDump(retllval, DtoType(rbase), DtoAlignment(nextbase),
@@ -983,7 +976,7 @@ DValue *DtoCallFunction(const Loc &loc, Type *resulttype, DValue *fnval,
     case TY::Taarray:
     case TY::Tpointer:
       if (tf->isref()) {
-        retllval = DtoBitCast(retllval, DtoType(rbase->pointerTo()));
+        retllval = DtoBitCast(retllval, DtoType(pointerTo(rbase)));
       } else {
         retllval = DtoBitCast(retllval, DtoType(rbase));
       }
@@ -1059,15 +1052,9 @@ DValue *DtoCallFunction(const Loc &loc, Type *resulttype, DValue *fnval,
     call->setCallingConv(gABI->callingConv(tf, iab.hasContext));
   }
   // merge in function attributes set in callOrInvoke
-#if LDC_LLVM_VER >= 1400
   auto attrbuildattribs = call->getAttributes().getFnAttrs();
   attrlist = attrlist.addFnAttributes(
       gIR->context(), llvm::AttrBuilder(gIR->context(), attrbuildattribs));
-#else
-  attrlist = attrlist.addAttributes(
-      gIR->context(), LLAttributeList::FunctionIndex,
-      llvm::AttrBuilder(call->getAttributes(), LLAttributeList::FunctionIndex));
-#endif
   call->setAttributes(attrlist);
 
   // Special case for struct constructor calls: For temporaries, using the
