@@ -24,6 +24,10 @@
 #include "ir/irfunction.h"
 #include "ir/irtypeclass.h"
 #include "ir/irtypestruct.h"
+#include <llvm/Analysis/ConstantFolding.h>
+#include <llvm/IR/Constant.h>
+
+using namespace dmd;
 
 /// Emits an LLVM constant corresponding to the expression (or an error if
 /// impossible).
@@ -75,8 +79,7 @@ public:
     }
 
     if (TypeInfoDeclaration *ti = e->var->isTypeInfoDeclaration()) {
-      result = DtoTypeInfoOf(e->loc, ti->tinfo, /*base=*/false);
-      result = DtoBitCast(result, DtoType(e->type));
+      result = DtoTypeInfoOf(e->loc, ti->tinfo);
       return;
     }
 
@@ -166,20 +169,17 @@ public:
 
     if (auto ts = t->isTypeSArray()) {
       const auto arrayLength = ts->dim->toInteger();
-      assert(arrayLength >= e->numberOfCodeUnits());
+      assert(arrayLength >= e->len);
       result = buildStringLiteralConstant(e, arrayLength);
       return;
     }
 
     llvm::GlobalVariable *gvar = p->getCachedStringLiteral(e);
-    LLConstant *arrptr = DtoGEP(gvar->getValueType(), gvar, 0u, 0u);
 
     if (t->ty == TY::Tpointer) {
-      result = DtoBitCast(arrptr, DtoType(t));
+      result = gvar;
     } else if (t->ty == TY::Tarray) {
-      LLConstant *clen =
-          LLConstantInt::get(DtoSize_t(), e->numberOfCodeUnits(), false);
-      result = DtoConstSlice(clen, arrptr, e->type);
+      result = DtoConstSlice(DtoConstSize_t(e->len), gvar);
     } else {
       llvm_unreachable("Unknown type for StringExp.");
     }
@@ -199,7 +199,7 @@ public:
     const dinteger_t byteOffset = e->e2->toInteger();
 
     LLConstant *llResult = nullptr;
-    const auto pointeeSize = pointeeType->size(e->loc);
+    const auto pointeeSize = size(pointeeType, e->loc);
     if (pointeeSize && byteOffset % pointeeSize == 0) { // can do a nice GEP
       LLConstant *llOffset = DtoConstSize_t(byteOffset / pointeeSize);
       if (negateOffset)
@@ -207,7 +207,6 @@ public:
       llResult = llvm::ConstantExpr::getGetElementPtr(DtoMemType(pointeeType),
                                                       llBase, llOffset);
     } else { // need to cast base to i8*
-      llBase = DtoBitCast(llBase, getVoidPtrType());
       LLConstant *llOffset = DtoConstSize_t(byteOffset);
       if (negateOffset)
         llOffset = llvm::ConstantExpr::getNeg(llOffset);
@@ -215,7 +214,7 @@ public:
           llvm::ConstantExpr::getGetElementPtr(getI8Type(), llBase, llOffset);
     }
 
-    return DtoBitCast(llResult, DtoType(e->type));
+    return llResult;
   }
 
   void visit(AddExp *e) override {
@@ -253,7 +252,7 @@ public:
                          e->type->toChars());
     LOG_SCOPE;
 
-    if (e->type->equivalent(e->e1->type)) {
+    if (equivalent(e->type, e->e1->type)) {
       if (!e->lwr && !e->upr) {
         result = toConstElem(e->e1, p);
         return;
@@ -287,11 +286,11 @@ public:
             StringExp *strexp = static_cast<StringExp*>(e1);
             size_t datalen = strexp->sz * strexp->len;
             Type* eltype = tb->nextOf()->toBasetype();
-            if (datalen % eltype->size() != 0) {
+            if (datalen % size(eltype) != 0) {
                 error("the sizes don't line up");
                 return e1->toConstElem(p);
             }
-            size_t arrlen = datalen / eltype->size();
+            size_t arrlen = datalen / size(eltype);
 #endif
       error(
           e->loc,
@@ -320,7 +319,7 @@ public:
       if (type->ty == TY::Tarray || type->ty == TY::Tdelegate) {
         value = DtoGEP(irg->getType(), value, 0u, 1u);
       }
-      result = DtoBitCast(value, DtoType(tb));
+      result = value;
     } else if (tb->ty == TY::Tclass && e->e1->type->ty == TY::Tclass &&
                e->e1->op == EXP::classReference) {
       auto cd = static_cast<ClassReferenceExp *>(e->e1)->originalClass();
@@ -338,7 +337,7 @@ public:
         // offset pointer
         instance = DtoGEP(DtoType(e->e1->type), instance, 0, i_index);
       }
-      result = DtoBitCast(instance, DtoType(tb));
+      result = instance;
     } else {
       goto Lerr;
     }
@@ -367,7 +366,7 @@ public:
       result = base;
     } else {
       const unsigned elemSize =
-          gDataLayout->getTypeStoreSize(DtoType(e->var->type));
+          gDataLayout->getTypeAllocSize(DtoType(e->var->type));
 
       IF_LOG Logger::println("adding offset: %llu (elem size: %u)",
                              static_cast<unsigned long long>(e->offset),
@@ -381,14 +380,10 @@ public:
       } else {
         // Offset isn't a multiple of base type size, just cast to i8* and
         // apply the byte offset.
-        auto i8 = LLType::getInt8Ty(gIR->context());
         result = llvm::ConstantExpr::getGetElementPtr(
-            i8, DtoBitCast(base, i8->getPointerTo()),
-            DtoConstSize_t(e->offset));
+            getI8Type(), base, DtoConstSize_t(e->offset));
       }
     }
-
-    result = DtoBitCast(result, DtoType(e->type));
   }
 
   //////////////////////////////////////////////////////////////////////////////
@@ -402,8 +397,7 @@ public:
 
     // address of global variable
     if (auto vexp = e->e1->isVarExp()) {
-      LLConstant *c = DtoConstSymbolAddress(e->loc, vexp->var);
-      result = c ? DtoBitCast(c, DtoType(e->type)) : nullptr;
+      result = DtoConstSymbolAddress(e->loc, vexp->var);
       return;
     }
 
@@ -425,18 +419,16 @@ public:
       // gep
       LLConstant *idxs[2] = {DtoConstSize_t(0), index};
       LLConstant *val = isaConstant(getIrGlobal(vd)->value);
-      val = DtoBitCast(val, DtoType(vd->type->pointerTo()));
       LLConstant *gep = llvm::ConstantExpr::getGetElementPtr(
           DtoType(vd->type), val, idxs, true);
 
-      // bitcast to requested type
       assert(e->type->toBasetype()->ty == TY::Tpointer);
-      result = DtoBitCast(gep, DtoType(e->type));
+      result = gep;
       return;
     }
 
     if (auto se = e->e1->isStructLiteralExp()) {
-      result = p->getStructLiteralConstant(se);
+      result = p->getStructLiteralGlobal(se);
       if (result) {
         IF_LOG Logger::cout()
             << "Returning existing global: " << *result << '\n';
@@ -448,12 +440,12 @@ public:
           llvm::GlobalValue::InternalLinkage, nullptr, ".structliteral");
       globalVar->setAlignment(llvm::MaybeAlign(DtoAlignment(se->type)));
 
-      p->setStructLiteralConstant(se, globalVar);
+      p->setStructLiteralGlobal(se, globalVar);
       llvm::Constant *constValue = toConstElem(se, p);
-      constValue = p->setGlobalVarInitializer(globalVar, constValue, nullptr);
-      p->setStructLiteralConstant(se, constValue);
+      globalVar = p->setGlobalVarInitializer(globalVar, constValue, nullptr);
+      p->setStructLiteralGlobal(se, globalVar);
 
-      result = constValue;
+      result = globalVar;
       return;
     }
 
@@ -501,7 +493,7 @@ public:
     assert(result);
 
     if (fd->tok != TOK::function_) {
-      auto contextPtr = getNullPtr(getVoidPtrType());
+      auto contextPtr = getNullPtr();
       result = LLConstantStruct::getAnon(gIR->context(), {contextPtr, result});
     }
   }
@@ -515,11 +507,6 @@ public:
 
     // extract D types
     Type *bt = e->type->toBasetype();
-    Type *elemt = bt->nextOf();
-
-    // build llvm array type
-    LLArrayType *arrtype =
-        LLArrayType::get(DtoMemType(elemt), e->elements->length);
 
     // dynamic arrays can occur here as well ...
     bool dyn = (bt->ty != TY::Tsarray);
@@ -538,18 +525,16 @@ public:
         llvm::GlobalValue::InternalLinkage, initval, ".dynarrayStorage");
     gvar->setUnnamedAddr(canBeConst ? llvm::GlobalValue::UnnamedAddr::Global
                                     : llvm::GlobalValue::UnnamedAddr::None);
-    llvm::Constant *store = DtoBitCast(gvar, getPtrToType(arrtype));
 
     if (bt->ty == TY::Tpointer) {
       // we need to return pointer to the static array.
-      result = store;
+      result = gvar;
       return;
     }
 
     // build a constant dynamic array reference with the .ptr field pointing
     // into store
-    LLConstant *globalstorePtr = DtoGEP(arrtype, store, 0u, 0u);
-    result = DtoConstSlice(DtoConstSize_t(e->elements->length), globalstorePtr);
+    result = DtoConstSlice(DtoConstSize_t(e->elements->length), gvar);
   }
 
   //////////////////////////////////////////////////////////////////////////////
@@ -575,8 +560,12 @@ public:
             LLConstant *c = toConstElem(elem, p);
             // extend i1 to i8
             if (c->getType()->isIntegerTy(1)) {
-              c = llvm::ConstantExpr::getZExt(c,
-                                              LLType::getInt8Ty(p->context()));
+              LLType *I8PtrTy = LLType::getInt8Ty(p->context());
+#if LDC_LLVM_VER < 1800
+              c = llvm::ConstantExpr::getZExt(c, I8PtrTy);
+#else
+              c = llvm::ConstantFoldCastOperand(llvm::Instruction::ZExt, c, I8PtrTy, *gDataLayout);
+#endif
             }
             varInits[e->sd->fields[i]] = c;
           }
@@ -598,14 +587,14 @@ public:
     DtoResolveClass(origClass);
     StructLiteralExp *value = e->value;
 
-    result = p->getStructLiteralConstant(value);
+    result = p->getStructLiteralGlobal(value);
     if (result) {
       IF_LOG Logger::cout() << "Using existing global: " << *result << '\n';
     } else {
       auto globalVar = new llvm::GlobalVariable(
           p->module, getIrType(origClass->type)->isClass()->getMemoryLLType(),
           false, llvm::GlobalValue::InternalLinkage, nullptr, ".classref");
-      p->setStructLiteralConstant(value, globalVar);
+      p->setStructLiteralGlobal(value, globalVar);
 
       std::map<VarDeclaration *, llvm::Constant *> varInits;
 
@@ -644,10 +633,10 @@ public:
 
       llvm::Constant *constValue =
           getIrAggr(origClass)->createInitializerConstant(varInits);
-      constValue = p->setGlobalVarInitializer(globalVar, constValue, nullptr);
-      p->setStructLiteralConstant(value, constValue);
+      globalVar = p->setGlobalVarInitializer(globalVar, constValue, nullptr);
+      p->setStructLiteralGlobal(value, globalVar);
 
-      result = constValue;
+      result = globalVar;
     }
 
     if (e->type->ty == TY::Tclass) {
@@ -666,7 +655,6 @@ public:
     }
 
     assert(e->type->ty == TY::Tclass || e->type->ty == TY::Tenum);
-    result = DtoBitCast(result, DtoType(e->type));
   }
 
   //////////////////////////////////////////////////////////////////////////////
@@ -703,13 +691,9 @@ public:
       // cast.
       // FIXME: Check DMD source to understand why two different ASTs are
       //        constructed.
-#if LDC_LLVM_VER >= 1200
       const auto elementCount = llvm::ElementCount::getFixed(elemCount);
-#else
-      const auto elementCount = llvm::ElementCount(elemCount, false);
-#endif
       result = llvm::ConstantVector::getSplat(
-          elementCount, toConstElem(e->e1->optimize(WANTvalue), p));
+          elementCount, toConstElem(optimize(e->e1, WANTvalue), p));
     }
   }
 
@@ -725,8 +709,7 @@ public:
       return;
     }
 
-    result = DtoTypeInfoOf(e->loc, t, /*base=*/false);
-    result = DtoBitCast(result, DtoType(e->type));
+    result = DtoTypeInfoOf(e->loc, t);
   }
 
   //////////////////////////////////////////////////////////////////////////////
