@@ -15,6 +15,8 @@
 #include "gen/tollvm.h"
 #include "llvm/IR/DerivedTypes.h"
 
+using namespace dmd;
+
 IrTypeFunction::IrTypeFunction(Type *dt, llvm::Type *lt, IrFuncTy irFty_)
     : IrType(dt, lt), irFty(std::move(irFty_)) {}
 
@@ -52,9 +54,9 @@ IrTypeDelegate *IrTypeDelegate::get(Type *t) {
 
   IrFuncTy irFty(tf);
   llvm::Type *ltf =
-      DtoFunctionType(tf, irFty, nullptr, Type::tvoid->pointerTo());
+      DtoFunctionType(tf, irFty, nullptr, pointerTo(Type::tvoid));
   llvm::Type *fptr = ltf->getPointerTo(gDataLayout->getProgramAddressSpace());
-  llvm::Type *types[] = {getVoidPtrType(), fptr};
+  llvm::Type *types[] = {getOpaquePtrType(), fptr};
   LLStructType *lt = LLStructType::get(gIR->context(), types, false);
 
   // Could have already built the type as part of a struct forward reference,
