@@ -8,6 +8,13 @@
  * Source: $(DRUNTIMESRC importc.h)
  */
 
+/**********************
+ * Silence noisy warnings for this file
+ */
+#ifdef __GNUC__
+#pragma GCC system_header
+#endif
+
 /**********************
  * For special casing ImportC code.
  */
@@ -33,6 +40,8 @@
 #define __attribute __attribute__
 #define __alignof _Alignof
 #define __vector_size__ vector_size
+#define __typeof typeof
+#define __typeof__ typeof
 
 /********************
  * Clang nullability extension used by macOS headers.
@@ -60,16 +69,27 @@
 #define __int32 int
 #define __int64 long long
 
+/* Linux builtin types */
+typedef unsigned short __uint16_t;
+typedef unsigned int __uint32_t;
+typedef unsigned long long __uint64_t;
+
 /*********************
  * Obsolete detritus
  */
 #define __cdecl
+#define __pascal
+
+/*********************
+ * DMC-specific extensions, https://digitalmars.com/ctg/pointers16.html
+ */
+#ifdef __DMC__
 #define __ss
 #define __cs
 #define __far
 #define __near
 #define __handle
-#define __pascal
+#endif
 
 /****************************
  * __extension__ is a GNU C extension. It suppresses warnings
@@ -79,7 +99,7 @@
 
 #define __builtin_isnan(x) isnan(x)
 #define __builtin_isfinite(x) finite(x)
-// IN_LLVM: replaced by symbol in __builtins.di
+// IN_LLVM: replaced by symbol in __importc_builtins.di
 //#define __builtin_alloca(x) alloca(x)
 
 /********************************
@@ -129,6 +149,7 @@
 #define __ptr64
 #define __unaligned
 #define _NO_CRT_STDIO_INLINE 1
+#define _stdcall __stdcall
 
 // This header disables the Windows API Annotations macros
 // Need to include sal.h to get the pragma once to prevent macro redefinition.
@@ -154,9 +175,9 @@
 
 #ifndef __aarch64__
 #define _Float32 float
-#define _Float32x float
+#define _Float32x double
 #define _Float64 double
-#define _Float64x double
+#define _Float64x long double
 #define _Float128 long double
 #define __float128 long double
 #endif
