@@ -10,6 +10,11 @@ set -euxo pipefail
 #
 # Afterwards, merging a new frontend+druntime is a matter of merging branch
 # `dmd-rewrite-stable` or some tag (`dmd-rewrite-v2.101.0`).
+#
+# NOTE: As it turns out, the rewrite unfortunately isn't stable across git versions.
+#       This process was started on Ubuntu 22, using git v2.34 and git-filter-repo v2.34 (22826b5a68b6).
+#       On Ubuntu 24 (git v2.43), this fails (to fast-forward push) with both default git-filter-repo v2.38 (ed61b4050b71) and v2.34.
+#       So you might need to use a Ubuntu 22 container for now to run this script successfully.
 
 initialize="${INITIALIZE:-0}" # set INITIALIZE=1 for the very first rewrite
 refs_prefix="dmd-rewrite-"
@@ -18,7 +23,10 @@ temp_dir="$(mktemp -d)"
 cd "$temp_dir"
 
 # generate message-filters.txt file for replacing GitHub refs in commit messages
-echo 'regex:(^|\s|\()#(\d{2,})==>\1dlang/dmd!\2' > message-filters.txt
+cat > message-filters.txt <<'EOF'
+regex:(^|\s|\()#(\d{2,})==>\1dlang/dmd!\2
+(cherry picked from commit 88d1e8fc37428b873f59d87f8dff1f40fbd3e7a3)==>(cherry picked from commit 8b9b481a322bdcbfdad38ba4ad74182742aef118)
+EOF
 
 # clone DMD monorepo
 git clone git@github.com:dlang/dmd.git dmd.tmp
@@ -50,6 +58,7 @@ git filter-repo --invert-paths \
   --path-regex '(compiler/)?src/dmd/frontend\.d' \
   --path-regex '(compiler/)?src/dmd/scan(elf|mach|mscoff|omf)\.d' \
   --path-regex '(compiler/)?src/dmd/lib(|elf|mach|mscoff|omf)\.d' \
+  --path 'compiler/src/dmd/lib/' \
   --path-regex '(compiler/)?src/dmd/link\.d' \
   --path-regex '(compiler/)?src/dmd/(e|s)2ir\.d' \
   --path-regex '(compiler/)?src/dmd/to(csym|ctype|cvdebug|dt|ir|obj)\.d' \
@@ -98,7 +107,7 @@ if [[ "$initialize" != 1 ]]; then
 fi
 
 # push prefixed master/stable branches and tags
-git push --tags ldc "${refs_prefix}master" "${refs_prefix}stable"
+git push --tags --atomic ldc "${refs_prefix}master" "${refs_prefix}stable"
 
 cd ../..
 rm -rf "$temp_dir"
